@@ -24,6 +24,7 @@
 #include "dialog.h"
 #include "slot.h"
 #include "statistics.h"
+#include "menu.h"
 
 /* These have been lifted straight from eog. */
 
@@ -79,6 +80,62 @@ enum {
   CURSOR_CLOSED
 };
 
+static gboolean cards_are_droppable (hslot_type hslot) {
+      SCM cardlist = SCM_EOL;
+      GList* temp;
+
+      if (!hslot) return FALSE;
+
+      for (temp = press_data->cards; temp; temp = temp->next)
+        cardlist = scm_cons(make_card(temp->data), cardlist);
+      return SCM_NFALSEP (cscmi_droppable_lambda (scm_long2num (press_data->hslot->id),
+                                                  cardlist,
+                                                  scm_long2num (hslot->id)));
+}
+
+static hslot_type find_drop_target(gint x, gint y) {
+  hslot_type new_hslot;
+  hslot_type retval = NULL;
+  gint i, new_cardid;
+  gint min_distance = G_MAXINT;
+
+  /* Find a target directly under the center of the card. */
+  slot_pressed(x + card_width / 2,
+               y + card_height / 2,
+	       &new_hslot, &new_cardid);
+
+  if (new_hslot && cards_are_droppable (new_hslot))
+    return new_hslot;
+
+  /* If that didn't work, look for a target at all 4 corners of the card. */
+  for (i = 0; i < 4; i++) {
+    slot_pressed(x + card_width * (i/2),
+                 y + card_height * (i%2),
+                 &new_hslot, &new_cardid);
+
+    if (!new_hslot)
+	continue;
+	
+    /* This skips corners we know are not droppable. */
+    if (!droppable_is_featured
+        || cards_are_droppable (new_hslot)) {
+      gint dx, dy, distance_squared;
+
+      dx = new_hslot->pixelx + new_cardid*new_hslot->pixeldx - x;
+      dy = new_hslot->pixely + new_cardid*new_hslot->pixeldy - y;
+
+      distance_squared = dx*dx + dy*dy;
+
+      if (distance_squared <= min_distance) {
+	retval = new_hslot;
+        min_distance = distance_squared;
+      }
+    }
+  }
+
+  return retval;
+}
+
 static void set_cursor (int cursor)
 {
   if (cursor == CURSOR_NONE)
@@ -121,6 +178,7 @@ void end_of_game_test() {
       refresh_screen ();
       if (game_won) { /* Check because we can undo lost games. */
 	game_in_progress = FALSE;
+	option_list_set_sensitive ();
 	update_statistics (TRUE, timer_get ());
       }
       show_game_over_dialog ();
@@ -132,15 +190,12 @@ void drop_moving_cards(gint x, gint y) {
   GList* temp;
   SCM cardlist = SCM_EOL;
   hslot_type hslot;
-  gint cardid, moved = 0;
+  gint moved = 0;
   gint width, height;
 
-  slot_pressed(x + card_width / 2 - press_data->xoffset, 
-	       y + card_height / 2 - press_data->yoffset, 
-	       &hslot, &cardid);
+  hslot = find_drop_target(x - press_data->xoffset, y - press_data->yoffset);
 
   if (hslot) {
-
     for (temp = press_data->cards; temp; temp = temp->next)
       cardlist = scm_cons(make_card(temp->data), cardlist);
     moved = SCM_NFALSEP (cscmi_button_released_lambda (scm_long2num (press_data->hslot->id),
@@ -151,7 +206,6 @@ void drop_moving_cards(gint x, gint y) {
   if (!moved) {
     hslot = press_data->hslot;
     add_cards_to_slot(press_data->cards, hslot);
-
     scm_c_eval_string ("(discard-move)");
   }
 
@@ -169,6 +223,52 @@ void drop_moving_cards(gint x, gint y) {
   if(moved) end_of_game_test();
 }
 
+static void highlight_drop_target(hslot_type hslot) {
+  static hslot_type old_hslot = NULL;
+
+  if (hslot != old_hslot) {
+    old_hslot = hslot;
+    gdk_window_hide(press_data->highlight_window);
+
+    if (cards_are_droppable(hslot)) {
+      GdkPixmap *droptarget_pixmap;
+
+      if (hslot->length) {
+      
+        hcard_type card;
+
+        gint delta = hslot->exposed ? hslot->exposed - 1 : 0;
+
+        int x = hslot->pixelx + delta * hslot->pixeldx;
+        int y = hslot->pixely + delta * hslot->pixeldy;
+
+        card = g_list_nth(hslot->cards, hslot->length - 1)->data;
+
+        droptarget_pixmap = get_droptarget_pixmap (card->suit, 
+						   card->value, 
+						   card->direction);
+
+        if (droptarget_pixmap) {
+          gdk_window_set_back_pixmap (press_data->highlight_window,
+                                      droptarget_pixmap, 0);
+          gdk_window_shape_combine_mask (press_data->highlight_window,
+                                    	 mask, 0, 0);
+          gdk_window_resize(press_data->highlight_window, 
+			    card_width, card_height);
+          gdk_window_clear(press_data->highlight_window);
+          gdk_window_move(press_data->highlight_window, x, y);
+          gdk_window_show(press_data->highlight_window);
+        }
+      } 
+    }
+
+  /* This ensures that the moving cards are on top.*/
+  gdk_window_show (press_data->moving_cards);
+  }
+}
+
+
+
 gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
 {
   hslot_type hslot;
@@ -178,6 +278,7 @@ gint button_press_event (GtkWidget *widget, GdkEventButton *event, void *d)
 
   if (!game_in_progress) {
     game_in_progress = TRUE;
+    option_list_set_sensitive ();
     timer_start ();
   }
 
@@ -296,6 +397,7 @@ gint button_release_event (GtkWidget *widget, GdkEventButton *event, void *d)
   if (event->button == 3 && press_data->status == STATUS_SHOW) {
     press_data->status = STATUS_NONE;
     gdk_window_hide(press_data->moving_cards);
+    highlight_drop_target(NULL);
     press_data->moving_pixmap = NULL;
     press_data->moving_mask = NULL;
     return TRUE;
@@ -312,6 +414,7 @@ gint button_release_event (GtkWidget *widget, GdkEventButton *event, void *d)
   switch (press_data->status) {
   case STATUS_IS_DRAG:
     press_data->status = STATUS_NONE;
+    highlight_drop_target(NULL);
     drop_moving_cards(event->x, event->y);
     break;
     
@@ -339,9 +442,17 @@ gint button_release_event (GtkWidget *widget, GdkEventButton *event, void *d)
 gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
 {
   if (press_data->status == STATUS_IS_DRAG) {
+    hslot_type hslot;
+
+    hslot = find_drop_target(event->x - press_data->xoffset,
+			     event->y - press_data->yoffset);
+    
+    highlight_drop_target(hslot);
+
     gdk_window_move(press_data->moving_cards,  
 		    event->x - press_data->xoffset,
 		    event->y - press_data->yoffset);
+
     gdk_window_clear(press_data->moving_cards);
     set_cursor (CURSOR_CLOSED);
   }
@@ -350,6 +461,7 @@ gint motion_notify_event (GtkWidget *widget, GdkEventMotion *event)
 	    abs(press_data->yoffset - event->y) > 2)) {
     press_data->status = STATUS_IS_DRAG;
     generate_press_data ();
+    refresh_screen ();
     take_snapshot();
     set_cursor (CURSOR_CLOSED);
   } else {
@@ -411,8 +523,7 @@ gint configure_event (GtkWidget *widget, GdkEventConfigure *event) {
   window_width = event->width;
   window_height = event->height;
 
-  surface =
-    gdk_pixmap_new (playing_area->window, window_width, window_height, -1);
+  surface = gdk_pixmap_new (playing_area->window, window_width, window_height, -1);
 
   rescale_cards ();
   refresh_screen();
@@ -423,7 +534,7 @@ gint configure_event (GtkWidget *widget, GdkEventConfigure *event) {
   settings = gtk_settings_get_default();
   g_object_get(G_OBJECT(settings),"gtk-double-click-time",&tmptime,NULL);
   dbl_click_time = tmptime/1000.0;
- 
+
   return FALSE;
 }
 
