@@ -27,6 +27,8 @@ G_BEGIN_DECLS
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
+#include "games-preimage.h"
+
 enum {
   GAMES_CARD_ACE         = 1,
   GAMES_CARD_TWO         = 2,
@@ -41,6 +43,7 @@ enum {
   GAMES_CARD_JACK        = 11,
   GAMES_CARD_QUEEN       = 12,
   GAMES_CARD_KING        = 13,
+  GAMES_CARD_ACE_HIGH    = 14,
   GAMES_CARDS_CLUBS      = 0,
   GAMES_CARDS_HEARTS     = 1,
   GAMES_CARDS_DIAMONDS   = 2,
@@ -59,6 +62,9 @@ typedef struct _GamesCardImages {
 
   gchar * themename;
 
+  GamesPreimage * preimage;
+  gboolean prescaled;
+
   GdkPixbuf * source;
   gint subwidth, subheight;
   gboolean prerendered;
